﻿using System.Collections.Generic;

namespace VectorSurvivors;


/// <summary>
/// Create one and fill it with upgradables. Then require any amount of upgrades.
/// Drop it right after usage and create a new one if needed.
/// </summary>
public class UpgradesProvider : IUpgradesProvider
{
    public bool CanBeDropped { get; set; } = true;
    private readonly HashSet<IUpgradable> _upgradables = new();
    
    public UpgradesProvider With(IUpgradable upgradable)
    {
        if(upgradable.CanBeUpgraded())
            _upgradables.Add(upgradable);
        
        return this;
    }

    public UpgradesProvider WithAll(WeaponsInventory inventory)
    {
        foreach (var weapon in inventory.Weapons)
        {
            With(weapon);
        }

        return this;
    }
    
    public List<PendingUpgrade> PickUpgrades(Player player, int amount, bool first = false)
    {
        List<PendingUpgrade> result = new();
        for (int i = 0; i < amount; i++)
        {
            var tier = first 
                ? new []{Tier.Legendary, Tier.Epic, Tier.Rare}.GetRandom() 
                : UpgradeUtils.GetRandomTier();

            var attempts = 10;
            PendingUpgrade upgrade = null;
            while (attempts > 0 && upgrade is null)
            {
                attempts--;
                upgrade = _upgradables.GetRandom().GetUpgrade(player, tier);
            }
            
            
            if(upgrade is not null)
                result.Add(upgrade);
        }
        
        return result;
    }
}