﻿using System;
using System.Collections.Generic;

namespace VectorSurvivors;

internal class WeaponsProvider : IUpgradesProvider, IUpgradable
{
    public string NameOverride => "Выберите оружие";
    /// <inheritdoc />
    public bool CanBeDropped => false;

    /// <inheritdoc />
    public List<PendingUpgrade> PickUpgrades(Player player, int amount, bool first = false)
    {
        List<PendingUpgrade> result = new();
        foreach (Type missingWeaponType in player.Weapons.GetMissingWeaponTypes())
        {
            var upgrade = GetWeaponUpgrade(player, missingWeaponType);
            if(upgrade is not null)
                result.Add(upgrade);
        }
        
        return result;
    }
    
    private PendingUpgrade GetWeaponUpgrade(Player player, Type type)
    {
        if (!player.Weapons.CanAddWeapon())
        {
            return null;
        }

        var newWeaponType = type;
        var tier = Tier.Weapon;
        
        var tmpNewWeapon = Game.Storage.Weapons.ByType[newWeaponType].Instantiate() as Weapon;
        var name = tmpNewWeapon.UpgradableName;
        var description = tmpNewWeapon.Description;
        var icon = tmpNewWeapon.Icon;
        
        tmpNewWeapon.QueueFree();
        var addWeaponAction = () =>
        {
            var newWeapon = Game.Storage.Weapons.ByType[newWeaponType].Instantiate() as Weapon;
            player.Weapons.AddWeapon(newWeapon);
        };

        return new PendingUpgrade(name, description, tier, this, addWeaponAction, icon);
    }

    /// <inheritdoc />
    public int Level { get; } = 0;

    /// <inheritdoc />
    public int MaxLevel { get; } = 1;

    /// <inheritdoc />
    public string UpgradableName { get; } = "Оружейная";

    /// <inheritdoc />
    public string Description { get; } = "Выберите оружие";

    /// <inheritdoc />
    public string StateInfo { get; } = "";

    /// <inheritdoc />
    public Texture2D Icon { get; } = null;

    /// <inheritdoc />
    public PendingUpgrade GetUpgrade(Player player, Tier tier)
    {
        throw new NotImplementedException();
    }

    /// <inheritdoc />
    public bool CanBeUpgraded()
    {
        throw new NotImplementedException();
    }

    /// <inheritdoc />
    public void OnUpgrade(PendingUpgrade upgrade)
    {
        //throw new System.NotImplementedException();
    }
}
