﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace VectorSurvivors;

public partial class Character
{
    protected Dictionary<Type, CharacterComponent> Components { get; private set; } = new();

    internal void ApplyPredefinedComponents()
    {
        foreach (var component in PredefinedComponents)
        {
            AddComponent(component);
        }
        PredefinedComponents = new();
    }
    
    public ReadOnlyDictionary<Type, CharacterComponent> GetComponents()
    {
        return Components.AsReadOnly();
    }
	
    public void AddComponent(CharacterComponent component)
    {
        Components.Add(component.GetType(), component);
        component.OnAdded(this);
    }
	
    public void RemoveComponent(CharacterComponent component)
    {
        Components.Remove(component.GetType());
        component.OnRemove(this);
        //component.Free();
    }

    public TComponent GetComponent<TComponent>() where TComponent : CharacterComponent
    {
        try
        {
            return Components[typeof(TComponent)] as TComponent;
        }
        catch
        {
            return null;
        }
    }
	
    public CharacterComponent GetComponent(Type type)
    {
        try
        {
            return Components[type];
        }
        catch
        {
            return null;
        }
    }
	
    public bool TryGetComponent<TComponent>(out TComponent component) where TComponent : CharacterComponent
    {
        component = GetComponent(typeof(TComponent)) as TComponent;
        return component is not null;
    }
}