﻿using System.Linq;
using Content;
using KludgeBox.Scheduling;

namespace VectorSurvivors;

[GlobalClass]
public partial class Enemy : Character
{
    [Export] public Area2D HurtBox { get; set; }
    [Export] public double ContactDamage { get; set; } = 1;
    [Export] public double HurtInterval { get; set; } = 0.1;

    private Cooldown _hurtCooldown;
    private Cooldown _returnToPlayerCooldown = new Cooldown(1);
    /// <inheritdoc />
    public override void _Ready()
    {
        base._Ready();
        _hurtCooldown = new Cooldown(HurtInterval);
        _hurtCooldown.Ready += ProcessHurt;
        _returnToPlayerCooldown.Ready += TryReturnToPlayer;
    }
    
    /// <inheritdoc />
    public override void _Process(double delta)
    {
        base._Process(delta);
        _hurtCooldown.Update(delta * UniversalAttackSpeedFactor);
        _returnToPlayerCooldown.Update(delta);
    }
    
    private void TryReturnToPlayer()
    {
        if(!World.Players.Any()) return;
        if(IsImmobilized) return;
        
        var moveTimeThreshold = 60;
        var distanceThreshold = MovementSpeed * moveTimeThreshold;
        var closestPlayer = World.Players.MinBy(ply => ply.GlobalPosition.DistanceSquaredTo(GlobalPosition));
        
        if (GlobalPosition.DistanceSquaredTo(closestPlayer) > distanceThreshold * distanceThreshold)
        {
            var directionToPlayer = closestPlayer.DirectionFrom(this);
            var radius = World.Instance.SpawnRadius;
            Position = closestPlayer.Position + directionToPlayer * radius;
            Smoother.Teleport();
        }
    }

    private void ProcessHurt()
    {
        var areas = HurtBox.GetOverlappingAreas();
        foreach (var area in areas)
        {
            if(area is not HitBox hitBox) continue;
            if(hitBox.GetParent() is not Player player) continue;

            var dmg = new Damage(ContactDamage * UniversalAttackDamageFactor, false, false, this);
            dmg.AddTag(DamageTag.Contact);
            player.TakeDamage(dmg);
            Audio2D.PlaySoundAt(Sfx.BassHit, player.Position);
        }
    }
}