using KludgeBox.Godot.Net;
using KludgeBox.Loggers;
using AbstractStorage = KludgeBox.Godot.AbstractStorage;
using Utils = KludgeBox.Godot.Utils;

namespace VectorSurvivors;

public partial class Game : AbstractStorage
{
    public static Game Instance { get; private set; }
    public static Storage Storage => Instance._storage;
    public static StateContainer State => Instance._state;
    public static UiContainer Overlay => Instance._overlay;

    private static Settings _settings;

    public static Settings Settings
    {
        get => _settings;
        set
        {
            _settings = value;
            Settings.Apply(_settings);
            _settings.Save();
        }
    }
    
    [Export] [NotNull] private Storage _storage { get; set; }
    [Export] [NotNull] private StateContainer _state { get; set; }
    [Export] [NotNull] private UiContainer _overlay { get; set; }

    private Game()
    {
        Instance = this;
        Settings = Settings.Load() ?? new Settings();
        if (Settings.MaximizeOnStart)
        {
            DisplayServer.WindowSetMode(DisplayServer.WindowMode.Maximized);
        }
        
        var services = new ServiceRegistry();
        services.RegisterServices();
        
        EventBus.RegisterListeners(services);
        
        // Логи также надобно записать в файлик.
        Log.AddLogger(new FileLogger("/custom-logs"));
        
        // Сканируем пакетики
        PacketRegistry.ScanPackets();
    }
    
    /// <inheritdoc />
    public override void _Ready()
    {
        base._Ready();
    }

    public static void Pause()
    {
        if(World.Instance.IsValid())
            World.Instance.ProcessMode = ProcessModeEnum.Disabled;
    }
    
    public static void Unpause()
    {
        if(World.Instance.IsValid())
            World.Instance.ProcessMode = ProcessModeEnum.Inherit;
    }
    
    public static Node ShowUi(PackedScene scene)
    {
        var ui = scene.Instantiate();
        if (ui is not IUiScene uiScene) return null;
        
        return ShowUi(uiScene);
    }
    
    public static Node ShowUi(IUiScene ui)
    {
        Overlay.PushState(ui);
        return ui as Node;
    }

    // Костыль, для того, чтобы кто-то не удалил игру из игры. Кидает ошибки при закрытии)0)00
    public override void _Notification(int what)
    {
        if (what == NotificationPredelete)
        {
            CancelFree();
        }

        if (what == NotificationUnparented)
        {
            Utils.SceneTree.Root.AddChild(this);
        }
    }

    
    public override void _Input(InputEvent @event)
    {
        if (@event.IsActionPressed(Keys.Pause))
        {
            if(PauseScreen.CanPause)
            {
                Overlay.PushState(Storage.Ui.PauseScreen.Instantiate() as PauseScreen);
                //ShowUi();
            }
            else
            {
                Overlay.PopState();
            }
        }
    }
}
