﻿using Newtonsoft.Json;

namespace VectorSurvivors;

public class Settings
{
    public static string FilePath { get; internal set; } = "user://settings.json";
    
    public float MasterVolume { get; set; } = 1;
    public float MusicVolume { get; set; } = 1;
    public float SoundVolume { get; set; } = 1;
    public bool MaximizeOnStart { get; set; } = false;

    
    public Settings GetCopy() => FromJson(ToString());
    
    public override string ToString()
    {
        return JsonConvert.SerializeObject(this);
    }
    
    public static Settings FromJson(string json)
    {
        return JsonConvert.DeserializeObject<Settings>(json);
    }
    
    internal static void Apply(Settings settings)
    {
        Audio2D.MasterVolume = settings.MasterVolume;
        Audio2D.MusicVolume = settings.MusicVolume;
        Audio2D.SoundsVolume = settings.SoundVolume;
    }

    public void Save()
    {
        var file = FileAccess.Open(FilePath, FileAccess.ModeFlags.Write);
        file.StoreString(ToString());
        file.Close();
    }
    
    public static Settings Load()
    {
        var file = FileAccess.Open(FilePath, FileAccess.ModeFlags.Read);
        if(file is null) return null;
        
        var json = file.GetAsText();
        file.Close();
        return FromJson(json);
    }
}