using System;
using System.Collections.Generic;
using System.Linq;

namespace VectorSurvivors;

public partial class StateContainer : Node
{
	public Node CurrentState { get; private set; }
	public Node RootState => StateStack.Last();

	public Stack<Node> StateStack { get; private set; } = new();
	
	
	/// <inheritdoc />
	public override void _Ready()
	{
		if(GetChildCount() > 0)
			CurrentState = GetChildren()[0]; // Хорошая попытка, пёс
	}

	public void PushState(Node newState, Action<Node> callback = null)
	{
		StateStack.Push(ChangeTo(newState, callback, false));
	}

	public Node PopState(Action<Node> callback = null)
	{
		if(StateStack.Count > 0)
			return ChangeTo(StateStack.Pop(), callback);

		return null;
	}
	

	/// <summary>
	/// Заменяет текущее состояние на новое и делает обратный вызов с новым состоянием. Возвращает ссылку на предыдущее состояние.
	/// </summary>
	/// <remarks>
	/// Возвращаемое предыдущее состояние уже находится в очереди на удаление.
	/// </remarks>
	/// <returns>Предыдущее состояние</returns>
	public Node ChangeTo(Node newState, Action<Node> callback = null, bool freeOldState = true)
	{
		var oldState = CurrentState;
		CurrentState = newState;
		if (oldState.IsValid())
		{
			RemoveChild(oldState);
			if (freeOldState)
				oldState?.QueueFree();
		}
		
		
		if(newState.IsValid())
			AddChild(CurrentState, true);
		
		callback?.Invoke(newState);
		return oldState;
	}
	
	/// <summary>
	/// Заменяет текущее состояние на новое и делает обратный вызов с новым состоянием. Возвращает ссылку на предыдущее состояние.
	/// </summary>
	/// <remarks>
	/// Возвращаемое предыдущее состояние уже находится в очереди на удаление.
	/// </remarks>
	/// <returns>Предыдущее состояние</returns>
	public Node ChangeTo(PackedScene newState, Action<Node> callback = null)
	{
		return ChangeTo(newState.Instantiate(), callback);
	}
}