using Content;

namespace VectorSurvivors;

public partial class Orb : Node2D
{
	[Export] public double MaxSpeed { get; private set; } = 1000;
	[Export] public Sprite2D Sprite { get; private set; }
	
	public Player PickedPlayer { get; set; }
	public double Velocity { get; set; }
	public bool IsPicked => PickedPlayer is not null;
	public Vector2 Direction { get; set; }
	

	/// <inheritdoc />
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
	}

	/// <inheritdoc />
	public override void _Process(double delta)
	{
		Position += Direction * Velocity * delta;
		if (PickedPlayer is not null)
		{
			Direction = PickedPlayer.DirectionFrom(this);
			Velocity += Mathf.Min(MaxSpeed * delta, MaxSpeed - Velocity);
			
			if (PickedPlayer.DistanceSquaredTo(this) <= Velocity * Velocity * 0.2 * delta)
			{
				Position = PickedPlayer.Position;
				Apply(PickedPlayer);
			}
		}
		else if(!Mathf.IsZeroApprox(Velocity))
		{
			Velocity -= Mathf.Min(MaxSpeed * delta * 0.5, MaxSpeed - Velocity);
			Velocity = Mathf.Max(Velocity, 0);
		}
		else if (Mathf.IsZeroApprox(Velocity))
		{
			Velocity = 0;
			ProcessMode = ProcessModeEnum.Disabled;
		}

		Velocity = Mathf.Min(Velocity, MaxSpeed);
	}

	public virtual void Pick(Player player)
	{
		ProcessMode = ProcessModeEnum.Inherit;
		PickedPlayer = player;
		MaxSpeed = player.MovementSpeed + MaxSpeed;
		Velocity = -MaxSpeed/3;
	}

	protected virtual void Apply(Player player)
	{
		QueueFree();
		World.Notify_EntityDespawned(this);
		Audio2D.PlaySoundAt(Sfx.Beep, Position, 0.5f).PitchVariation(0.2f);
	}
}