using Godot;
using System;
using System.Collections.Generic;
using VectorSurvivors;

public partial class CreditsScreen : CanvasLayer, IUiScene
{
	public static readonly string MusicCreditsPath = "res://Credits/Music.txt";
	[Export] [NotNull] public VBoxContainer CreditsContainer { get; private set; }
	[Export] [NotNull] public Button QuitButton { get; private set; }

	/// <inheritdoc />
	public bool PausesGame { get; } = true;

	/// <inheritdoc />
	public bool PreventClose { get; } = false;
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
		QuitButton.Pressed += OnQuit;
		var file = FileAccess.Open(MusicCreditsPath, FileAccess.ModeFlags.Read);
		var text = file.GetAsText();
		file.Close();

		var lines = text.Split("\n");
		var parsedLines = new List<(string, string, string)>();
		foreach (var line in lines)
		{
			var parts = line.Split("::");
			var id = parts[0].Trim();
			var name = parts[1].Trim();
			var url = parts[2].Trim().Substring(1, parts[2].Trim().Length - 2);
			parsedLines.Add((id, name, url));
		}
		
		foreach (var (id, name, url) in parsedLines)
		{
			var creditsItem = Game.Storage.Ui.CreditsItem.Instantiate() as CreditsItem;
			creditsItem.Initialize(id, name, url);
			CreditsContainer.AddChild(creditsItem);
		}
	}

	private void OnQuit()
	{
		QueueFree();
	}
}
