using System;

namespace VectorSurvivors;

public partial class UpgradeCard : PanelContainer
{
    [Export] [NotNull] public TextureRect Icon { get; private set; }
    [Export] [NotNull] public Label TierLabel { get; private set; }
    [Export] [NotNull] public Label UpgradableInfo { get; private set; }
    [Export] [NotNull] public Label NameLabel { get; private set; }
    [Export] [NotNull] public Label DescriptionLabel { get; private set; }
    [Export] [NotNull] public Button UpgradeButton { get; private set; }
    
    /// <inheritdoc />
    public override void _Ready()
    {
        NotNullChecker.CheckProperties(this);
    }

    public void Initialize(PendingUpgrade upgrade)
    {
        UpgradeButton.Pressed += upgrade.ApplyUpgrade;

        TierLabel.Text = $"{upgrade.Tier}";
        TierLabel.LabelSettings.FontColor = upgrade.Color;

        var target = upgrade.Upgradable;
        if(!String.IsNullOrWhiteSpace(target.UpgradableName))
        {
            UpgradableInfo.Text = $"{target.UpgradableName} - ур. {target.Level}/{target.MaxLevel}";
        }
        else
        {
            UpgradableInfo.Text = String.Empty;
        }

        var styleBox = GetThemeStylebox("panel") as StyleBoxFlat;
        styleBox.BorderColor = upgrade.Color;
        
        styleBox = UpgradeButton.GetThemeStylebox("normal") as StyleBoxFlat;
        styleBox.BorderColor = upgrade.Color;
        
        
        if(upgrade.Icon.IsValid())
            Icon.Texture = upgrade.Icon;
        
        UpgradeButton.AddThemeColorOverride("font_color", upgrade.Color);
        
        NameLabel.Text = upgrade.Name;
        DescriptionLabel.Text = upgrade.Description;
    }
}