namespace VectorSurvivors;

public partial class PauseScreen : CanvasLayer, IUiScene
{
	public static bool CanPause => !_shown && Game.State.CurrentState is Gameplay;
	
	public bool PausesGame => true;

	public bool PreventClose => false;
	
	[Export] [NotNull] public HBoxContainer UpgradablesContainer { get; private set; }
	
	[Export] [NotNull] public Button SettingsButton { get; private set; }
	[Export] [NotNull] public Button QuitButton { get; private set; }

	private static bool _shown = false;
	
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
		Game.Pause();
		_shown = true;
		SettingsButton.Pressed += OnSettings;
		QuitButton.Pressed += OnQuit;
		TreeExiting += OnClose;
		
		var upgradables = World.LocalPlayer.GetUpgradables();

		foreach (var upgradable in upgradables)
		{
			var card = Game.Storage.Ui.UpgradableCard.Instantiate() as UpgradableCard;
			card.Initialize(upgradable);
			UpgradablesContainer.AddChild(card);
		}
	}

	public void OnClose()
	{
		//QueueFree();
		if(IsQueuedForDeletion())
			_shown = false;
	}
	

	private void OnQuit()
	{
		QueueFree();
		_shown = false;
		World.ExitWorld();
	}
	
	private void OnSettings()
	{
		Game.ShowUi(Game.Storage.Ui.SettingsScreen);
	}

}