using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace VectorSurvivors;

public partial class PlayerHud : Control
{
	[ExportGroup("Bars")]
	[Export] [NotNull] public TwoColoredBar HpBar { get; private set; }
	[Export] [NotNull] public TwoColoredBar XpBar { get; private set; }
	
	
	[ExportGroup("Labels")]
	[Export] [NotNull] public Label Level { get; private set; }
	[Export] [NotNull] public Label Enemies { get; private set; }
	[Export] [NotNull] public Label GameTime { get; private set; }
	[Export] [NotNull] public Label Difficulty { get; private set; }
	[Export] [NotNull] public Label EnemyHp { get; private set; }
	
	
	[ExportGroup("FPS & TPS")]
	[Export] [NotNull] public Label Fps { get; private set; }
	[Export] [NotNull] public Label Tps { get; private set; }
	
	private readonly Stopwatch _physicsStopwatch = new();
	private readonly Queue<double> _deltas = new();
	
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
	}

	public override void _Process(double delta)
	{
		var player = World.LocalPlayer;
		if (player == null) return;

		Enemies.Text = $"Врагов: {World.Enemies.Count}";
		Level.Text = $"Уровень: {player.Level}";
		GameTime.Text = $"Время игры: {TimeSpan.FromSeconds(World.GameTime):h\\:mm\\:ss\\.f}";
		Difficulty.Text = $"Сложность: {World.Difficulty:N1}";
		EnemyHp.Text = $"Здоровье врагов +{(World.Instance.EnemyHpOffsetPerDifficulty * World.Difficulty):N0}";
		
        
		HpBar.CurrentUpperValue = player.Health;
		double hpBarValueDelta = Mathf.Clamp(HpBar.CurrentLowerValue - HpBar.CurrentUpperValue, 
			0, Math.Max(HpBar.MaxValue - HpBar.CurrentUpperValue, 0));
		double hpBarValueDeltaDecrease = HpBar.MaxValue * 0.25 * delta;
		
		HpBar.CurrentLowerValue = player.Health + hpBarValueDelta - hpBarValueDeltaDecrease;
		HpBar.MaxValue = player.MaxHealth;
		HpBar.Label.Text = "";//$"Здоровье: {player.Health:N0} / {player.MaxHealth:N0}";
		
		XpBar.CurrentUpperValue = player.Xp;
		XpBar.MaxValue = player.RequiredXp;
		XpBar.Label.Text = "";//$"Опыт: {player.Xp:N0} / {player.RequiredXp:N0}";
		Fps.Text = $"FPS: {Engine.GetFramesPerSecond():N0}";
	}

	public override void _PhysicsProcess(double delta)
	{
		var measuredDelta = _physicsStopwatch.Elapsed.TotalSeconds;
        
		_deltas.Enqueue(measuredDelta);
		if (_deltas.Count >= 120)
		{
			var tps = _deltas.Average();
			Tps.Text = $"TPS: {1/tps:N0}";
			_deltas.Dequeue();
		}
		_physicsStopwatch.Restart();
	}
}