namespace VectorSurvivors;

public partial class SettingsScreen : CanvasLayer, IUiScene
{

	public bool PausesGame => true;
	public bool PreventClose => false;
	
	[Export] [NotNull] public HSlider MasterVolumeSlider { get; private set; }
	[Export] [NotNull] public HSlider MusicVolumeSlider { get; private set; }
	[Export] [NotNull] public HSlider SoundVolumeSlider { get; private set; }
	
	[Export] [NotNull] public Button SaveButton { get; private set; }
	[Export] [NotNull] public Button CancelButton { get; private set; }
	[Export] [NotNull] public CheckBox MaximizeCheckBox { get; private set; }
	
	private static Settings _preserved = null;
	private bool _saved = false;
	
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
		_preserved = Game.Settings.GetCopy();
		
		MasterVolumeSlider.Value = Audio2D.MasterVolume;
		MusicVolumeSlider.Value = Audio2D.MusicVolume;
		SoundVolumeSlider.Value = Audio2D.SoundsVolume;
		MaximizeCheckBox.ButtonPressed = _preserved.MaximizeOnStart;
		
		MasterVolumeSlider.ValueChanged += value => Audio2D.MasterVolume = (float)value;
		MusicVolumeSlider.ValueChanged += value => Audio2D.MusicVolume = (float)value;
		SoundVolumeSlider.ValueChanged += value => Audio2D.SoundsVolume = (float)value;
		
		SaveButton.Pressed += OnSave;
		CancelButton.Pressed += OnCancel;

		TreeExited += OnCancel;
	}

	private void OnSave()
	{
		_saved = true;
		var settings = new Settings();
		
		settings.MasterVolume = (float)MasterVolumeSlider.Value;
		settings.MusicVolume = (float)MusicVolumeSlider.Value;
		settings.SoundVolume = (float)SoundVolumeSlider.Value;
		settings.MaximizeOnStart = MaximizeCheckBox.ButtonPressed;
		
		Game.Settings = settings;
		QueueFree();
	}

	
	private void OnCancel()
	{
		if (!_saved)
		{
			Game.Settings = _preserved;
			QueueFree();
		}
	}
}