using Godot;
using System;
using System.Linq;
using VectorSurvivors;

public partial class WorldSetupMenu : CanvasLayer, IUiScene
{
	[Export] [NotNull] public VBoxContainer PropertiesContainer { get; private set; }
	[Export] [NotNull] public Button StartButton { get; private set; }
	[Export] [NotNull] public Button CancelButton { get; private set; }
	
	private World _world;
	private Gameplay _gameplay;
	
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
		StartButton.Pressed += OnStart;
		CancelButton.Pressed += OnCancel;
		
		_gameplay = Game.Storage.States.Gameplay.Instantiate() as Gameplay;
		_world = _gameplay.World;
		var propertiesControls = _world.GetWorldProperties().Select(prop => prop.AsControl());
		foreach (var propertyControl in propertiesControls)
		{
			PropertiesContainer.AddChild(propertyControl);
		}
	}

	private void OnStart()
	{
		Audio2D.StopMusic(2);
		Game.Overlay.ClearStack();
		Game.State.ChangeTo(_gameplay);
		QueueFree();
	}

	private void OnCancel()
	{
		QueueFree();
	}

	/// <inheritdoc />
	public bool PausesGame { get; } = true;

	/// <inheritdoc />
	public bool PreventClose { get; } = false;
}
