using Content;
using KludgeBox.Godot.Net;
using AbstractStorage = KludgeBox.Godot.AbstractStorage;

namespace VectorSurvivors;

public partial class MainMenu : AbstractStorage, IUiScene
{
	public bool PausesGame => false;
	public bool PreventClose => true;
	
	[Export] [NotNull] public Button CreateGameButton { get; private set; }
	[Export] [NotNull] public Button CreateSandboxButton { get; private set; }
	[Export] [NotNull] public Button ConnectToGameButton { get; private set; }
	[Export] [NotNull] public Button SettingsButton { get; private set; }
	[Export] [NotNull] public Button CreditsButton { get; private set; }
	[Export] [NotNull] public Button QuitButton { get; private set; }
	[Export] [NotNull] public ColorRect FadingForeground { get; private set; }
	

	/// <inheritdoc />
	public override void _Ready()
	{
		CreateGameButton.Pressed += OnCreateGame;
		CreateSandboxButton.Pressed += OnCreateSandbox;
		ConnectToGameButton.Pressed += OnConnectToGame;
		SettingsButton.Pressed += OnSettings;
		CreditsButton.Pressed += OnCredits;
		QuitButton.Pressed += OnQuit;
		
		FadingForeground.Visible = true;
		var tween = FadingForeground.CreateTween();
		tween.TweenProperty(FadingForeground, "modulate:a", 0, 2);

		TreeExiting += () =>
		{
			if (tween.IsValid()) tween.Kill();
		};

		PlayMusic();
	}

	private void PlayMusic()
	{
		Audio2D.PlayMusic(Music.MainBgm, 0.5f).Finished += PlayMusic;
	}


	private void OnCreateGame()
	{
		// Подготавливаем сетку перед запуском сервера
		Audio2D.StopMusic(2);
		//Network.CreateServer(DefaultNetworkSettings.Port);
		Game.State.ChangeTo(Game.Storage.States.Gameplay);
		QueueFree();
	}
	
	private void OnCreateSandbox()
	{
		Game.ShowUi(Game.Storage.Ui.WorldSettings);
	}
	
	private void OnConnectToGame()
	{
		// Подготавливаем сетку перед запуском клиента
		Network.ConnectToRemoteServer(DefaultNetworkSettings.Host, DefaultNetworkSettings.Port);
		Game.State.ChangeTo(Game.Storage.States.Gameplay);
	}
	
	private void OnSettings()
	{
		Game.ShowUi(Game.Storage.Ui.SettingsScreen);
	}
	private void OnCredits()
	{
		Game.ShowUi(Game.Storage.Ui.CreditsScreen);
	}

	private void OnQuit()
	{
		GetTree().Quit();
	}
	
}