using System;

namespace VectorSurvivors;

public partial class Projectile : Node2D
{
	public event Action<Character> CharacterHit;
	public event Action<Area2D> AreaEntered;
	public event Action<Area2D> AreaExited;
	public event Action TimeExpired; 
	
	[Export] public double Speed { get; set; } = 0;
	[Export] public double Lifetime { get; set; } = 0;
	[Export] public double Damage { get; set; } = 0;
	[Export] public double CritChance { get; set; } = 0; // 0 - no chance; 1 = 100% chance; 1,5 - 100% crit once and also 50% crin one more time
	[Export] public double CritMultiplier { get; set; } = 0;
	
	[Export] [NotNull] public Sprite2D Sprite { get; set; }
	[Export] [NotNull] public Area2D HurtBox { get; set; }
	
	public Weapon Weapon { get; set; }
	public Player Shooter { get; set; }
	
	public double TimeToLive { get; set; } = 0;
	public override void _Ready()
	{
		NotNullChecker.CheckProperties(this);
		HurtBox.AreaEntered += area => AreaEntered?.Invoke(area);
		HurtBox.AreaExited += area => AreaExited?.Invoke(area);
		HurtBox.BodyEntered += body =>
		{
			if (body is Character character)
			{
				CharacterHit?.Invoke(character);
			}
		};
	}

	public virtual void Configure(Weapon weapon, Player player)
	{
		Shooter = player;
		Weapon = weapon;
	}
	
	public virtual Damage GetDamage()
	{
		return new Damage(0);
	}
	
	public virtual double CalculateCritMultiplier()
	{
		var chance = CritChance;
		var multiplier = 1d;

		while (chance > 0)
		{
			var nextChance = Mathf.Min(chance, 1);
			if (Rand.Chance(nextChance))
			{
				multiplier *= CritMultiplier;
			}
			chance -= nextChance;
		}

		return multiplier;
	}

	// Called every frame. 'delta' is the elapsed time since the previous frame.
	public override void _Process(double delta)
	{
		
	}

	/// <inheritdoc />
	public override void _PhysicsProcess(double delta)
	{
		Position += this.Right() * Speed * delta;
		TimeToLive += delta;
		if (TimeToLive >= Lifetime)
		{
			QueueFree();
			TimeExpired?.Invoke();
		}
	}
}