﻿namespace VectorSurvivors;

public partial class PiercingProjectile : Projectile
{
    [Export] public bool IsPiercing { get; set; }
    
    public static PiercingProjectile Create()
    {
        return Game.Storage.Projectiles.PiercingProjectile.Instantiate<PiercingProjectile>();
    }
        
    /// <inheritdoc />
    public override void _Ready()
    {
        base._Ready();
        AreaEntered += OnAreaEntered;
    }

    /// <inheritdoc />
    public override Damage GetDamage()
    {
        var critMultiplier = CalculateCritMultiplier();
        var damage = Damage * critMultiplier;
        var isCrit = critMultiplier > 1;

        return new Damage(damage, isCrit, false, Shooter);
    }

    /// <inheritdoc />
    public override void Configure(Weapon weapon, Player player)
    {
        base.Configure(weapon, player);
    }

    /// <inheritdoc />
    public override void _Process(double delta)
    {
        base._Process(delta);
    }

    /// <inheritdoc />
    public override void _PhysicsProcess(double delta)
    {
        base._PhysicsProcess(delta);
    }
    
    protected virtual void OnAreaEntered(Area2D area)
    {
        if (area is HitBox hitbox)
        {
            var parent = hitbox.GetParent();
            if(parent is not Enemy enemy) return;
            enemy.TakeDamage(GetDamage());
            if (!IsPiercing)
            {
                QueueFree();
            }
        }
    }
}