using System.Linq;

namespace VectorSurvivors;

public partial class RicochetProjectile : PiercingProjectile
{
	[Export] public GpuParticles2D Trail { get; private set; }
	[Export] public int RicochetRange { get; set; } = 150;
	[Export] public int RicochetsLeft { get; set; } = 2;
	
	private bool _canDamagePlayers = false;

	public new static RicochetProjectile Create()
	{
		return Game.Storage.Projectiles.RicochetProjectile.Instantiate<RicochetProjectile>(); 
	}
	public override void _Ready()
	{
		base._Ready();
		TreeExiting += () =>
		{
			Trail.CancelFree();
			var dummy = Trail.Drop();
			Trail.Emitting = false;
			Trail.Destruct(Trail.Lifetime);
		};
	}
	
	protected override void OnAreaEntered(Area2D area)
	{
		if (area is HitBox hitbox)
		{
			var parent = hitbox.GetParent();
			if (parent is not Character character) return;

			if (character is Enemy || (_canDamagePlayers && parent is Player))
			{
				character.TakeDamage(GetDamage());
				_canDamagePlayers = true;
				RicochetsLeft--;
			}
			
			if (!IsPiercing && RicochetsLeft <= 0)
			{
				QueueFree();
			}

			if (RicochetsLeft > 0 && character is not Player)
			{
				Enemy target = World.Enemies.Where(enemy => enemy != character && enemy.DistanceSquaredTo(this) < RicochetRange * RicochetRange).GetRandomOrDefault();
				if (target is not null)
				{
					Rotation = (target.Position - Position).Angle();
				}
			}
			
		}
	}
}