using Content;

namespace VectorSurvivors;

public partial class RicochetWeapon : ProjectileWeapon
{
	[Export] public int RicochetRange { get; set; } = 150;
	[Export] public int Ricochets { get; set; } = 3;
	public override string StateInfo => base.StateInfo + $"\nОтскоки: {Ricochets}\n" +
	                                    $"Радиус отскока: {RicochetRange}\n";

	public override Texture2D Icon => Game.Storage.Textures.RicochetWeaponIcon;
	/// <inheritdoc />
	public override void _Ready()
	{
		base._Ready();
		UpgradesRepository.Add(UpgradeRicochetCount, Tier.Rare, Tier.Epic, Tier.Legendary);
		UpgradesRepository.AddToAll(UpgradeRicochetRange);
	}

	public override RicochetProjectile CreateProjectile()
	{
		return RicochetProjectile.Create();
	}

	public override void PlayShootSound()
	{
		Audio2D.PlaySoundAt(Sfx.LightPlasmaShot, GlobalPosition, 0.3f).PitchVariation(0.2f);
	}
	
	protected override double GetCritMultiplierUpgradeOffset(Player player, Tier tier)
	{
		return (int)tier switch
		{
			1 => 5,
			2 => 10,
			3 => 15,
			4 => 25,
			5 => 50
		};
	}
	
	public override void ConfigureProjectile(Player player, PiercingProjectile projectile)
	{
		base.ConfigureProjectile(player, projectile);
		var ricochetProjectile = projectile as RicochetProjectile;

		ricochetProjectile.RicochetsLeft = Ricochets;
		ricochetProjectile.RicochetRange = RicochetRange;
	}
	
	protected override double GetCritChanceUpgradeOffset(Player player, Tier tier)
	{
		return (int)tier switch
		{
			1 => 1,
			2 => 2,
			3 => 3,
			4 => 5,
			5 => 10
		};
	}
	
	protected override double GetDamageUpgradeMultiplier(Player player, Tier tier)
	{
		return UpgradeUtils.SwitchTier(tier,
			common: 0.02,
			uncommon: 0.05,
			rare: 0.1,
			epic: 0.2,
			legendary: 0.3
		);
	}
	
	protected override double GetReloadTimeUpgradeMultiplier(Player player, Tier tier)
	{
		return UpgradeUtils.SwitchTier(tier,
			common: 0.02,
			uncommon: 0.05,
			rare: 0.1,
			epic: 0.2,
			legendary: 0.3);
	}
	
	private PendingUpgrade UpgradeRicochetCount(Player player, Tier tier)
	{
		var ricochets = (int)UpgradeUtils.SwitchTier(tier, 0, 0, 1, 2, 4);
		var newRicochets = Ricochets + ricochets;
		var name = $"Отскоки +{ricochets:N0}";
		var description = $"Снаряд отскакивает\n{ricochets} -> {ricochets:N0} {TextUtils.Declension(newRicochets, "раз", "раза", "раз")}";
		var icon = Icon;
		
		return new PendingUpgrade(name, description, tier, this, ()=> Ricochets += ricochets, icon);
	}

	private PendingUpgrade UpgradeRicochetRange(Player player, Tier tier)
	{
		var baseRange = 25;
		var tieredRange = baseRange * tier;
		var icon = Icon;
		var newRange = RicochetRange + tieredRange;
		
		var name = $"Радиус отскока +{tieredRange:N0}";
		var description = $"Увеличивает радиус обнаружения цели для отскока\n{tieredRange:N0} -> {newRange:N0}";
		
		return new PendingUpgrade(name, description, tier, this, ()=> RicochetRange += tieredRange, icon);
	}
}