﻿namespace VectorSurvivors;



public partial class Weapon : Node2D, IUpgradable
{
    [Export] public string UpgradableName { get; protected set; } = null;
    [Export(PropertyHint.MultilineText)] public string Description { get; protected set; } = null;
    [Export] public int Level { get; protected set; }
    [Export] public int CustomMaxLevel { get; protected set; } = -1;
    [Export] public double ReloadTime { get; protected set; }
    [Export] public double BaseDamage { get; protected set; }
    public virtual Texture2D Icon => null;
    
    
    public virtual string StateInfo => null;
    
    public int MaxLevel => CustomMaxLevel > 0 ? CustomMaxLevel : World.Instance.MaxWeaponLevel;
    
    protected readonly UpgradesRepository UpgradesRepository = new();
    protected bool IsReady { get; set; } = false;

    /// <inheritdoc />
    public virtual PendingUpgrade GetUpgrade(Player player, Tier tier)
    {
        return UpgradesRepository.GetAnyForTier(tier)?.Invoke(player, tier);
    }

    /// <inheritdoc />
    public override void _Ready()
    {
        base._Ready();
        IsReady = true;
    }


    /// <inheritdoc />
    public bool CanBeUpgraded()
    {
        return Level < MaxLevel;
    }

    /// <inheritdoc />
    public void OnUpgrade(PendingUpgrade upgrade)
    {
        Level++;
    }

    public virtual void OnEquipped(Player player)
    {
        // do nothing
    }

    public virtual void OnUnequipped(Player player)
    {
        // do nothing
    }

    public virtual void UpdateAttack(Player player, double delta)
    {
        // do nothing
    }
}